#include "SDL.h" 
#include "SDL_image.h" 
#include "SDL_mixer.h" 
#include "SDL_ttf.h" 
#include "globals.h"
#include "system.h"
#include <string> 
#include <fstream>
#include <cstdio>

using namespace std;


SDL_Surface *load_image(string filename) {
	// Write log file
	write_log("Loading image \"" + filename + "\"... ");

    // The image that's loaded
    SDL_Surface* loadedImage = NULL;
    
    // The optimized image that will be used
    SDL_Surface* optimizedImage = NULL;
    
    // Load the image using SDL_image
    loadedImage = IMG_Load(filename.c_str());
    
    // If the image loaded
    if(loadedImage != NULL){


        // Create an optimized image
        optimizedImage = SDL_DisplayFormat(loadedImage);
        
		//Map the color key (white)
		Uint32 colorkey = SDL_MapRGB(optimizedImage->format, 0xFF, 0xFF, 0xFF); 

		//Set all pixels of color R 0xFF, G 0xFF, B 0xFF to be transparent
		SDL_SetColorKey(optimizedImage, SDL_SRCCOLORKEY, colorkey); 

        // Free the old image
        SDL_FreeSurface(loadedImage);
    }
    
	// Write log file
	write_log("done", true);

    // Return the optimized image
    return optimizedImage;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void apply_surface(int x, int y, SDL_Surface* source, SDL_Surface* destination) {
    // Make a temporary rectangle to hold the offsets
    SDL_Rect offset;
    
    // Give the offsets to the rectangle
    offset.x = x;
    offset.y = y;

	// Blit the surface
	SDL_BlitSurface(source, NULL, destination, &offset); 
} 

////////////////////////////////////////////////////////////////////////////////////////////////////////

bool init() {
	// Write log file
	write_log("Initializing SDL subsystems... ");

    // Initialize all SDL subsystems
    if(SDL_Init( SDL_INIT_EVERYTHING ) == -1) {
		write_log("error", true);
        return false;    
    }

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Initializing SDL_TTF... ");

	//Initialize SDL_ttf
    if(TTF_Init() == -1) {
		// Write log file
		write_log("error", true);
        return false;    
    }

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Setting video mode to 800x600... ");

    // Set up the screen
    screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_SWSURFACE);
    
    // If there was an error in setting up the screen
    if(screen == NULL) {
		// Write log file
		write_log("error", true);
        return false;    
    }

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Initializing SDL_Mixer... ");

	//Initialize SDL_mixer 
	if(Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 4096) == -1) {
		// Write log file
		write_log("error", true);
		return false; 
	} 

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Setting window title... ");

    // Set the window caption
    SDL_WM_SetCaption("Black Orbit 2", NULL);

	// Write log file
	write_log("done", true);
    
    // If everything initialized fine
    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

bool load_files() {
    // Load the image
    pos			= load_image("Data\\x.png");
	title		= load_image("Data\\blackorbit.png");
	background	= load_image("Data\\bg.png"); 
    
    // If there was an error in loading the image
    if(pos == NULL) {
        return false;    
    }

	// If there was an error in loading the image
    if(background == NULL) {
        return false;    
    }

	// If there was an error in loading the image
    if(title == NULL) {
        return false;    
    }

	// Write log file
	write_log("Opening font file... ");

	// Open the font 
	font = TTF_OpenFont("Data\\LiberationSans-Bold.ttf", 13); 

	// If there was an error in loading the font 
	if(font == NULL) {
		return false; 
		// Write log file
		write_log("error", true);
	} 

	// Write log file
	write_log("done", true);
    
	// Write log file
	write_log("Loading music... ");

	// Load the music
	music = Mix_LoadMUS("Data\\beat.wav"); 

	// If there was a problem loading the music
	if(music == NULL) {
		// Write log file
		write_log("error", true);
		return false;
	} 

	// Write log file
	write_log("done", true);

    // If everything loaded fine
    return true;    
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void clean_up() {

	// Write log file
	write_log("Cleaning surfaces... ");

    // Free the image
    SDL_FreeSurface(pos);
	SDL_FreeSurface(background);
	SDL_FreeSurface(message);
	SDL_FreeSurface(title);
    
	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Closing font... ");

	// Close the font that was used
	TTF_CloseFont(font);

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Quitting SDL_TTF... ");

	// Quit SDL_ttf 
	TTF_Quit(); 

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Cleaning music buffer... ");

	// Free the sound effects
	Mix_FreeMusic(music); 

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Quitting SDL_Mixer... ");

	//Quit SDL_mixer
	Mix_CloseAudio(); 

	// Write log file
	write_log("done", true);

	// Write log file
	write_log("Quitting SDL... ");

    // Quit SDL
    SDL_Quit();    

	// Write log file
	write_log("done", true);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char* args[]) {
	// Delete the old log file
	remove("log.txt");

	// Make sure the program waits for a quit
    bool quit = false;

	// Initialize
    if(init() == false) {
        return 1;    
    }
    
    // Load the files
    if(load_files() == false) {
        return 1;    
    }

	// Render the text
    message = TTF_RenderText_Solid(font, "Dies ist eine Testnachricht.", textColor);
    
    // If there was an error in rendering the text
    if(message == NULL) {
        return 1;    
    }

	int x_pos_pointer = 0;
	int y_pos_pointer = 0;
	int x_pos_pointer_max = 800 - 32;
	int y_pos_pointer_max = 600 - 32;
	int x_pos_pointer_min = 0;
	int y_pos_pointer_min = 0;


	int x_pos_title = 200;
	int x_pos_title_max = 800;
	int x_pos_title_min = -400;
	int x_pos_title_speed = 3;

	// Get the keystates 
	Uint8 *keystates = SDL_GetKeyState(NULL); 

	// While the user hasn't quit 
	while( quit == false) { 
		if(keystates[ SDLK_UP ]) {
			if(y_pos_pointer >= y_pos_pointer_min) {
				y_pos_pointer--;
			}
		}

		if(keystates[ SDLK_DOWN ]) {
			if(y_pos_pointer <= y_pos_pointer_max) {
				y_pos_pointer++;
			}
		}

		if(keystates[ SDLK_LEFT ]) {
			if(x_pos_pointer >= x_pos_pointer_min) {
				x_pos_pointer--;
			}
		}

		if(keystates[ SDLK_RIGHT ]) {
			if(x_pos_pointer <= x_pos_pointer_max) {
				x_pos_pointer++;
			}
		}


		// While there's an event to handle
		while(SDL_PollEvent(&event)) {            
			// If the user has Xed out the window
			if (event.type == SDL_QUIT) {
                // Quit the program
                quit = true;
            }    
        }

		// Apply the background to the screen
		apply_surface(0, 0, background, screen); 

		// Apply the message to the screen 
		apply_surface(200, 50, message, screen); 

		// Apply the pos to the screen 
		apply_surface(x_pos_pointer, y_pos_pointer, pos, screen); 

		// Apply the title to the screen 
		apply_surface(x_pos_title, 200, title, screen); 

		x_pos_title += x_pos_title_speed;

		if(x_pos_title >= x_pos_title_max) {
			x_pos_title = x_pos_title_min;
		}


		// If there is no music playing
        if(Mix_PlayingMusic() == 0) {
            // Play the music
            if(Mix_PlayMusic(music, -1) == -1) {
                return 1;
            }    
        }


		// Update the screen 
		if(SDL_Flip(screen) == -1) {
			return 1; 
		} 

		SDL_Delay(1);
    }

	// Free the surface and quit SDL 
	clean_up(); 
	
	// Return 
	return 0; 
} 